import Layout from "./layout.js";
import Iframe from "./iframe.js";

const App = (function() {
  let instance;

  class SingletonApp {
    constructor() {
      if (!instance) {
        instance = this;
        this.init();
      }
      return instance;
    }

    init() {
      this.layout = new Layout();
      this.iframe = new Iframe();
      this.initEventListeners();
    }

    async #sendInitData() {
      await this.streamData(['init', '*']);
    }

    async #reconnect() {
        const response = await this.streamData(['reconnect', '*']);
        if (response.redirect) {
          window.location.reload();
        }
    }
    #sendWooSyncComplete() {
        this.sendMessage({message: 'woo-sync-complete'}, '*');
    }
   async #sendHeaderData() {
      await this.streamData(['refresh-token','*']);
    }
    #refreshListener() {
      document.addEventListener('DOMContentLoaded', function () {
        if (performance.navigation.type === performance.navigation.TYPE_RELOAD) {
          localStorage.setItem('refresh', 'true');
          // Your code to run only on page refresh
        }
      });
    }

    #closeListener() {
      window.addEventListener('unload', (event) => {
        if (!localStorage.getItem('refresh')) {
          this.resetLocalStorage();
        }
      });
    }
    initEventListeners() {

      this.#refreshListener();
      this.#closeListener();
      window.addEventListener('message', async (event) => {
        // todo: check if the origin is the origin of the react app
        const { message, data } = event.data;
        switch (message) {
          case 'init':
            await this.#sendInitData();
            break;
          case 'reconnect':
            await this.#reconnect();
            break;
          case 'get-integrations':
            await this.streamData( ["integrations", "*"]);
            break;
          case 'get-integrations-status':
            await this.streamData( ["get-integrations-status", "*"]);
            break;
          case 'get-forms':
            await this.streamData( ["forms", "*"]);
            break;
          case 'disconnect-plugin':
            await this.streamData( ["disconnect-plugin", "*",], 'POST', {sourceNameId: data});
            break;
          case 'connect-plugin':
            await this.streamData (["connect-plugin", "*",],'POST', {sourceNameId: data});
            break;
          case 'activate-plugin':
            await this.streamData (["activate-plugin", "*",],'POST', {sourceNameId: data});
            break;
          case 'create-plugin-forms':
            await this.streamData(["create-plugin-forms", "*",], 'POST');
            break;
          case 'sync-woo':
            this.streamDataNoBlock(["sync-woo", "*",]);
            break;
          case 'set-current-path':
            localStorage.setItem('route',data);
            break;
          case 'refresh-token':
            await this.#sendHeaderData();
            break;
          default:
            console.warn("Unknown request from plugin iframe (react app) : ", message);
        }
      });
    }

    getStreamNames() {
      return [
        ["integrations", "*"],
        ["forms", "*"],
        ["deactivate-plugin", "*"],
        ["activate-plugin", "*"],
      ];
    }

    // function to stream data to the iframe, could be a small subset of data or all data depending on the parameters
    async streamData(streamNames = null, method = 'GET', body = null, headers = null, allRequiredData = false, waitForResponse = true) {
      const data = await this.fetchStreamData(streamNames[0], method, body, headers, waitForResponse);
      data.streamName = streamNames?.[0];

      this.sendMessage(data, '*'); 
      return data; 
    }

    async streamDataNoBlock(streamNames = null, method = 'GET', body = null, headers = null, allRequiredData = false) {
      this.fetchStreamData(streamNames[0], method, body, headers, false);
      const data = { streamName: streamNames?.[0] };
      this.sendMessage(data, '*');

      return { message: 'success', data: 'Data streamed successfully' };
    }

    sendMessage(message, targetOrigin = '*') {
      this.iframe.sendMessage(message, targetOrigin);
    }


    resetLocalStorage() {
      localStorage.removeItem('refresh');
      localStorage.removeItem('route');
    }

    async fetchStreamData(stream, method = 'GET', body = null, headers = null, waitForResponse = true) {
      const baseUrl = `${data.baseUrl}/wp-json/emp-plugin/v1/${stream}`;
      const options = {
        url: baseUrl,
        method: method,
        ...!waitForResponse && { timeout: 0 },
        success: (response) => {
          return response;
        },
        error: (error) => {
          this.sendMessage({ message: 'error', data: `Failed to fetch data for stream ${stream}` });
        }
      };

      if (method.toUpperCase() === 'POST') {
        if (headers) {
          options.headers = headers;
        }
        options.data = body;
      }
      if (!waitForResponse) {
        return jQuery.ajax(options);
      } else {
        return await jQuery.ajax(options);
      }
    }
  }
    return SingletonApp;
})();

const app = new App();
