class CF7ViewedForm {
    constructor() {
        this.init();
    }

    init() {
        document.addEventListener('DOMContentLoaded', () => {
            this.listenForViewedForms();
        });

        // Listen for changes in the DOM to detect dynamically added popups
        this.observePopupChanges();
    }

    // Function to check if an element has a specific ancestor tag or attribute
    hasAncestorTagOrAttribute(element, tagName, attributeName, attributeValue) {
        while (element) {
            if (element.tagName && element.tagName.toLowerCase() === tagName.toLowerCase()) {
                return true;
            }
            if (element.hasAttribute(attributeName) && element.getAttribute(attributeName) === attributeValue) {
                return true;
            }
            element = element.parentElement;
        }
        return false;
    }

    listenForViewedForms() {
        const forms = document.querySelectorAll('div.wpcf7');
        forms.forEach(form => {
            this.processForm(form);
        });
    }

    processForm(form) {
        let formLocation = 'page';

        if (this.hasAncestorTagOrAttribute(form, 'header', 'data-elementor-type', 'header')) {
            formLocation = 'header';
        } else if (this.hasAncestorTagOrAttribute(form, 'footer', 'data-elementor-type', 'footer')) {
            formLocation = 'footer';
        } else if (this.hasAncestorTagOrAttribute(form, '', 'data-elementor-type', 'popup')) {
            formLocation = 'popup';
        }

        let formId = form.getAttribute('id').replace('wpcf7-f', '');
        // Extract only numeric characters from the form ID
        formId = formId.match(/\d+/)[0];

        const formElement = form.querySelector('form');
        if (!formElement) {
            return;
        }

        const hiddenInput = document.createElement('input');
        hiddenInput.type = 'hidden';
        hiddenInput.name = 'formLocation';
        hiddenInput.value = formLocation;
        formElement.appendChild(hiddenInput);

        const hiddenCount = document.createElement('input');
        hiddenCount.type = 'hidden';
        hiddenCount.name = 'hiddenCount';
        hiddenCount.value = '0';
        formElement.appendChild(hiddenCount);

        let formViewed = false;

        // Handle form visibility and hook calling
        const checkFormVisibility = () => {
            if (this.isElementInViewport(form) && !formViewed) {
                this.callServerHook(formId, formLocation);
                formViewed = true;
            }
        };

        // Initial check in case the form is already visible
        checkFormVisibility();

        // Listen for scroll events to detect when the form enters the viewport
        window.addEventListener('scroll', checkFormVisibility);

        // Additionally, observe the form's visibility in case the popup becomes visible later
        const visibilityObserver = new IntersectionObserver((entries) => {
            entries.forEach(entry => {
                if (entry.isIntersecting && !formViewed) {
                    this.callServerHook(formId, formLocation);
                    formViewed = true;
                }
            });
        });

        visibilityObserver.observe(form);

        formElement.addEventListener('submit', () => {
            if (hiddenCount.value !== "0") {
                this.callServerHook(formId, formLocation);
            }

            const num = parseInt(hiddenCount.value) + 1;
            hiddenCount.value = num.toString();
        });
    }

    isElementInViewport(element) {
        const rect = element.getBoundingClientRect();
        return (
            rect.top < (window.innerHeight || document.documentElement.clientHeight) &&
            rect.left < (window.innerWidth || document.documentElement.clientWidth) &&
            rect.bottom > 0 &&
            rect.right > 0
        );
    }

    observePopupChanges() {
        const observer = new MutationObserver((mutationsList) => {
            for (let mutation of mutationsList) {
                if (mutation.type === 'childList') {
                    mutation.addedNodes.forEach(node => {
                        if (node instanceof HTMLElement && node.matches('.wpcf7')) {
                            // The form has been added, now process it
                            this.processForm(node);
                        }
                    });
                }
            }
        });

        observer.observe(document.body, {
            childList: true,
            subtree: true
        });
    }

    callServerHook(formId, formLocation) {
        const baseUrl = `${data.baseUrl}/wp-json/emp-plugin/v1/cf7/viewed`;
        jQuery.ajax({
            url: baseUrl,
            data: {
                formId: formId,
                formLocation: formLocation
            },
            method: 'POST',
            success: function (response) {
            },
            error: function (error) {
                console.error('Error calling server hook:', error);
            }
        });
    }
}

const cf7ViewedForm = new CF7ViewedForm();
