class ElementorViewedForm {
    constructor() {
        this.init();
    }

    init() {
        document.addEventListener('DOMContentLoaded', () => {
            this.listenForViewedForms();
        });
        this.observePopupChanges();
    }

    hasAncestorTagOrAttribute(element, tagName, attributeName, attributeValue) {
        while (element) {
            if (element.tagName && element.tagName.toLowerCase() === tagName.toLowerCase()) {
                return true;
            }
            if (element.hasAttribute(attributeName) && element.getAttribute(attributeName) === attributeValue) {
                return true;
            }
            element = element.parentElement;
        }
        return false;
    }

    listenForViewedForms() {
        const formContainers = document.querySelectorAll('.elementor-widget-container');
        formContainers.forEach(formContainer => {
            this.processFormContainer(formContainer);
        });
    }

    processFormContainer(formContainer) {
        const formIdInput = formContainer.querySelector('input[name="form_id"]');
        if (!formIdInput) {
            return;
        }
        const formId = formIdInput.value;
        let formLocation = 'page';

        if (this.hasAncestorTagOrAttribute(formContainer, 'header', 'data-elementor-type', 'header')) {
            formLocation = 'header';
        } else if (this.hasAncestorTagOrAttribute(formContainer, 'footer', 'data-elementor-type', 'footer')) {
            formLocation = 'footer';
        } else if (this.hasAncestorTagOrAttribute(formContainer, '', 'data-elementor-type', 'popup')) {
            formLocation = 'popup';
        }

        const formElement = formContainer.querySelector('form');
        if (!formElement) {
            return;
        }

        const hiddenInput = document.createElement('input');
        hiddenInput.type = 'hidden';
        hiddenInput.name = 'formLocation';
        hiddenInput.value = formLocation;
        formElement.appendChild(hiddenInput);

        const hiddenCount = document.createElement('input');
        hiddenCount.type = 'hidden';
        hiddenCount.name = 'hiddenCount';
        hiddenCount.value = '0';
        formElement.appendChild(hiddenCount);

        let formViewed = false;

        const checkFormVisibility = () => {
            if (this.isElementInViewport(formContainer) && !formViewed) {
                this.callServerHook(formId, formLocation);
                formViewed = true;
            }
        };

        checkFormVisibility();
        window.addEventListener('scroll', checkFormVisibility);

        const visibilityObserver = new IntersectionObserver((entries) => {
            entries.forEach(entry => {
                if (entry.isIntersecting && !formViewed) {
                    this.callServerHook(formId, formLocation);
                    formViewed = true;
                }
            });
        });

        visibilityObserver.observe(formContainer);

        formElement.addEventListener('submit', () => {
            const num = parseInt(hiddenCount.value) + 1;
            hiddenCount.value = num.toString();

            this.callServerHook(formId, formLocation);
        });
    }

    isElementInViewport(element) {
        const rect = element.getBoundingClientRect();
        return (
            rect.top < (window.innerHeight || document.documentElement.clientHeight) &&
            rect.left < (window.innerWidth || document.documentElement.clientWidth) &&
            rect.bottom > 0 &&
            rect.right > 0
        );
    }

    observePopupChanges() {
        const observer = new MutationObserver((mutationsList) => {
            for (let mutation of mutationsList) {
                if (mutation.type === 'childList') {
                    mutation.addedNodes.forEach(node => {
                        if (node instanceof HTMLElement && node.matches('.elementor-popup-modal')) {
                            const formContainers = node.querySelectorAll('.elementor-widget-container');
                            formContainers.forEach(formContainer => {
                                this.processFormContainer(formContainer);
                            });
                        }
                    });
                }
            }
        });

        observer.observe(document.body, {
            childList: true,
            subtree: true
        });
    }

    callServerHook(formId, formLocation) {
        const baseUrl = `${data.baseUrl}/wp-json/emp-plugin/v1/elementor/viewed`;
        jQuery.ajax({
            url: baseUrl,
            data: {
                formId: formId,
                formLocation: formLocation
            },
            method: 'POST',
            success: function (response) {
            },
            error: function (error) {
                console.error('Error calling server hook:', error);
            }
        });
    }
}

const elementorViewedForm = new ElementorViewedForm();
