<?php

namespace Send\Classes\Integrations\Forms;

use connect\Client;
use Send\Classes\Database\Database;
use Send\Classes\Database\Models\Page;
use Send\Classes\Database\Models\Post;
use Send\Classes\Database\Models\PostMeta;
use Send\Classes\Integrations\Integrations;
use Send\Classes\Traits\Submission;
use Send;

class Cf7 extends Forms
{
	public $sourceName = 'cf7';
	public $name = 'Contact Form 7';
    private static $instance = null;


	public function __construct()
    {
		$this->resetState();
	}
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

	function resetState() {
		$this->pluginFile = 'contact-form-7/wp-contact-form-7.php';
		if (!function_exists('is_plugin_active')) {
			require_once(ABSPATH . '/wp-admin/includes/plugin.php');
		}
		if (self::isPluginActive()) {
			parent::__construct($this->name);
		}
		if (isset($this->pluginData)) {
			$this->pluginData['logoPath'] = plugins_url( '/app/Assets/images/cf7.svg', Send::$pluginDir );
		}
		$this->init();
	}

	function init() {
		if (self::isPluginActive()) {
			$this->active = true;
			// The Contact Form 7 plugin is active, proceed with initializing the action
			add_action('wpcf7_before_send_mail', [$this, 'onFormSubmit']);
		}
	}

	/*
		Handle the viewed form event
		@param array $payload
		@return string
	*/
	public static function handleViewedForm($payload) {
		$formIdParts = explode('-', $payload['formId']);
		$desiredValue = $formIdParts[0];
//		error_log("Viewed form ID: " . $desiredValue);

		return wp_json_encode($desiredValue);
	}

	/*
		Handle the abandoned form event
		@param array $payload
		@return string

	*/
	public static function handleAbandonedForm($payload) {
		$formIdParts = explode('-', $payload['formId']);
		$desiredValue = $formIdParts[0];
//		error_log('Abandoned form ID: ' . $desiredValue);
		return wp_json_encode($desiredValue);
	}
	/*
		Get form data on form submit
		@param array $fields
		@return array
	*/
	public static function isPluginActive()
	{
		if (!function_exists('is_plugin_active')) {
			require_once(ABSPATH . '/wp-admin/includes/plugin.php');
		}
		return is_plugin_active('contact-form-7/wp-contact-form-7.php');
	}

	public function onFormSubmit($fields = null)
	{
		$submission = \WPCF7_Submission::get_instance();
		if ($submission) {
			$form = $submission->get_contact_form();
			$fields = $submission->get_posted_data();
			$payload = [$form->id, $form->title(), $fields];
//			error_log("Submission Detected - form ID: " . $form->id);
			Submission::sendFormSubmissionData($payload,$this->sourceName);
		}
		return [];
	}

	/*
		Get all forms fields
		@return array
	*/
	static function getAllFormsFields()
	{
		// Get all Contact Form 7 forms
		$cf7Forms = get_posts(array(
			'post_type' => 'wpcf7_contact_form',
		));
		// Initialize an array to store form data
		$formsData = [];
		// Loop through each form
		foreach ($cf7Forms as $cf7Form) {
			$formId = $cf7Form->ID;
			$formName = $cf7Form->post_title;
			$formContent = $cf7Form->post_content;
			$fields = array();
			// Regular expression to find form fields
			preg_match_all('/\[([^\]]+)\]/', $formContent, $matches);
			if (isset($matches[1]) && !empty($matches[1])) {
				$fieldId = 1;
				// Loop through each form field
				foreach ($matches[1] as $field) {
					// Regular expression to find field name
					$pattern = '/\*?[\s:]([\w-]+)/';
					$fieldType = explode(' ', $field)[0];
					// Check if the field is mandatory
					$mandatory = false;
					// Check if the field is mandatory ( if includes * )
					if (strpos($fieldType, '*') !== false) {
						$mandatory = true;
						// Remove asterisk from field name
						$fieldType = str_replace('*', '', $fieldType);
					}
					preg_match_all($pattern, $field, $matches);
					if (!empty($matches[1])) {
						// Get the field name
						$fieldName = $matches[1][0];
						// Add field data to the fields array
						$fields[] = array(
							'type' => $fieldType,
							'fieldName' => strval($fieldName),
							'orderInForm' => $fieldId,
							'isRequired' => $mandatory,
						);
						$fieldId++;
					}
				}
			}
			// Store form fields in the formsData array using the form ID as the key
			$formsData[$formId] = [
				'source' => 'cf7',
				'formId' => strval($formId),
				'formName' => $formName,
				'fields' => $fields,
				'numberOfFields' => count($fields),
			];
		}
		return $formsData;
	}

	/*
		Get hash (long id ) of a form by id
		@param int $id
		@return array
	*/
	public static function getHash($idArr)
	{
		$hashes = [];
		foreach ($idArr as $id) {
			$query = PostMeta::where('post_id', $id)->where('meta_key', '_hash');
			$res = Database::execute($query);
			$hash = Database::field('meta_value', $res);
			$hashes[$id] = $hash ? substr($hash[0], 0, 7) : null;
		}
		return $hashes;
	}



	/*
		Get all (short) ids of forms
		@return array
	*/
	public static function getAllIds()
	{
		$query = Post::Select('ID')->where('post_type', 'wpcf7_contact_form');
		$res = Database::execute($query);
		return Database::field('ID', $res);
	}

	/*
		Get pages of a form by id
		@param int $id
		@return array
	*/
	public static function getPagesOfForm($form)
	{
		$query = Page::whereIn('post_type', ['page', 'elementor_library'])
			->where('post_content', 'like', '%[contact-form-7 id="' . $form . '"%')
			->where('post_status','publish')
		->select('ID', 'post_status', 'post_name', 'guid', 'post_type');
		$res = Database::execute($query);
		return $res;
	}

	/*
		* Get all forms and their pages
		 * @return array
	*/
	static function getTitlesOfHashArr($hashArr)
	{
		$titlesTable = [];
		foreach ($hashArr as $hash) {
			$id = PostMeta::where('meta_key', '_hash')
				->where('meta_value', 'LIKE', "%$hash%")->pluck('post_id');
			$title = Post::whereIn('ID', $id)->pluck('post_title')->first();
			$titlesTable[$hash] = $title;
		}
		return $titlesTable;
	}

	public static function getAllForms()
	{
		$formsIds = self::getAllIds();
		$hashArr = self::getHash($formsIds);
		$titlesTable = self::getTitlesOfHashArr($hashArr);
		$formsData = [];

		foreach ($hashArr as $hash) {
			$formPagesData = []; // Initialize an array to store data about pages where the form is used
			$res = self::getPagesOfForm($hash);
			foreach ($res as $page) {
				$pageData = [
					'pageId' => strval($page->ID),
					'name' => $page->post_name,
					'permalink' => get_permalink($page->ID),
					'pageType' => Integrations::getPageType($page->ID),
				];
				$formPagesData[] = $pageData; // Add data about the current page to the formPagesData array
			}

			// Add form hash and associated page data to the formsData array
			$formsData[] = [
				'formId' => array_search($hash, $hashArr),
				'formHash' => $hash,
				'formName' => $titlesTable[$hash],
				'pages' => $formPagesData,
			];
		}
		return $formsData;
	}

	public static function getAllData ($syncMethod = 'form')
	{
		if (!self::isPluginActive()) {
			return [];
		}
		$cf7FormsFields = self::getAllFormsFields();
		$cf7Forms = self::getAllForms();

		$cf7Payload = [];
		foreach ($cf7Forms as $form) {
			$formId = strval($form['formId']);
			$form['formId'] = $formId;
			$form['fields'] = $cf7FormsFields[$formId]['fields'];
			$form['source'] = [
				'method' => $syncMethod,
				'name' => 'cf7',
			];
			$cf7Payload[] = $form;
		}

		return $cf7Payload ? $cf7Payload : [];
	}

}
