<?php
namespace Send\Classes\Integrations;
use connect\Client;
use Send\Classes\Database\Database;
use Send\Classes\Database\Models\Post;
use Send\Classes\Database\Models\PostMeta;
use Send\Classes\Integrations\Forms\Cf7;
use Send\Classes\Integrations\Forms\ElementorForms;
use Send\Classes\Integrations\Forms\WpForms;

class Integrations
{
	// metadata of the plugin (will be populated from the plugin file)
	public $active = false;
	// metadata of the plugin (will be populated from the plugin file)
	public $pluginData = [];
	// the plugin file (will be populated from the plugin file)
	public $pluginFile = '';

	// supported plugins
	public static $supportedPlugins = [
		"Contact Form 7" => "cf7",
		"Elementor" => "elementor",
		"WPForms Lite" => "wpforms", //TODO: Make sure we don't want to differ between the various types (Lite/ Pro, etc..)
		"WPForms Pro" => "wpforms",
		"WPForms" => "wpforms",
		"WooCommerce" => "woocommerce",
	];

	public function __construct($pluginName) {
		$this->pluginData =  get_plugin_data( WP_PLUGIN_DIR . '/' . $this->pluginFile );
        $this->pluginData['Name'] = $pluginName;
	}


	static function getPageType($pageId)
	{
		if (isset($pageId)) {
			$res = Post::where('id',$pageId);
			$page = Database::execute($res);
			$page = $page[0];
			if ($page->post_type === 'page') {
				return 'page';
			} elseif ($page->post_type === 'elementor_library') {
				$res  = PostMeta::where('post_id', $page->ID)->where('meta_key', '_elementor_template_type');
				$meta = Database::execute($res);
				if (isset($meta)) {
					return $meta[0]->meta_value;
				}
			}
		}
		// default. Should not reach here
		return 'page';
	}
	public static function getPluginFromAppState($sourceNameId)
	{
		$app = \Send::getInstance();
		if (isset($app->integrations[$sourceNameId])) {
			return $app->integrations[$sourceNameId];
		}
	}

	public static function connect($data)
	{
		if (isset($data['sourceNameId']) && strlen($data['sourceNameId']) > 0) {
			return ["sourceNameId" => $data['sourceNameId'], "success" => true, "isConnected" => true];
		} else {
			throw new \Exception('Attempt to connect plugin with no name!');
		}
	}

	public static function disconnect($data) {
		if (isset($data['sourceNameId']) && strlen($data['sourceNameId']) > 0) {
			return ["sourceNameId" => $data['sourceNameId'], "success" => true, "isConnected" => false];
		} else {
			throw new \Exception('Attempt to disconnect plugin with no name!');
		}
	}

	/*
		this function is responsible for getting all the supported  && active integrations
		it's accessing the app instance and getting the integrations from it
		then it's looping through the integrations and getting the data of the supported plugins
		used for full sync of the installed & activated plugins (for example on send plugin activation or any page save / delete)
	*/
	static function getAllSupportedActiveIntegrations()
	{
        error_reporting(0);
		$app = \Send::getInstance();
		$integrations = $app->integrations;
		$integrationsPayload = [];
		$getPluginsStatus = self::getPlugins();
		if (!$getPluginsStatus) {
            return [];
        }
		// Create an associative array for quick lookup of plugin active status
		$pluginsStatus = [];
		foreach ($getPluginsStatus->plugins as $plugin) {
            if (!isset($plugin->name)) {
                continue;
            }
			$pluginsStatus[$plugin->name]['isConnected'] = $plugin->isConnected;
//			error_log($plugin->name . ' is active: ' . $plugin->isConnected);
			$pluginsStatus[$plugin->name]['isActive'] = $plugin->isActive;
//			error_log($plugin->name . ' is active: ' . $plugin->isActive);
		}

		foreach ($integrations as $name => $integration) {
			// Prevent cases of empty plugin data
			if (isset($integration->pluginData["Name"]) && strlen($integration->pluginData["Name"]) > 0) {
				// Allow only supported plugins
				if (!isset(self::$supportedPlugins[$integration->pluginData["Name"]])) {
					continue;
				}
                $isConnected = $pluginsStatus[$integration->pluginData["Name"]]['isConnected'];
				$integrationsPayload[] = [
					'name'        => $integration->pluginData["Name"],
					'by'          => $integration->pluginData["AuthorName"],
					'isConnected'    => $isConnected ? $isConnected : false,
					'version'     => $integration->pluginData['Version'],
					'sourceNameId' => $integration->sourceName,
					'description' => $integration->pluginData['Description'],
					'RequiresWP'  => $integration->pluginData['RequiresWP'],
					'RequiresPHP' => $integration->pluginData['RequiresPHP'],
					'logoPath'    => $integration->pluginData['logoPath'],
                    'activated' => true
				];
			}
		}
		return $integrationsPayload;
	}

	/*
	 *  this following function is responsible for getting all the supported active integrations
	 it's accessing the app instance and getting the integrations from it
	 then it's looping through the integrations and getting the data of the supported plugins
	 and returning the data of the supported plugins that are not active only
	 (can be used on partial sync of installed plugins and on full sync)
	*/
	static function getSupportedInstalledOnlyIntegrations()
	{
		// the plugin list
		$allPlugins = get_plugins();
		// the supported plugins that are not active
		$supportedOnlyInstalledPlugins = [];
		// get the instance of the app
		$app = \Send::getInstance();
		// init the integrations
		$app->initIntegrations();
		// get the integrations
		$integrations = $app->integrations;
		// loop through the plugins
		foreach ($allPlugins as $plugin) {
			// the plugin name
			$pluginName = $plugin['Name'];
			// convert short name  / plugin name to our standard name
			// check if the plugin is supported
			if (isset(self::$supportedPlugins[$pluginName])) {
				$integrationName = self::$supportedPlugins[$pluginName];
					// the plugin instance (from plugin.php )
				$appPlugin = $integrations[$integrationName];
				// check if the plugin is not active but supported + installed
				if (isset($appPlugin) && !$appPlugin->active) {
//					error_log('Plugin is supported & installed but not active: ' . $pluginName);
					$supportedOnlyInstalledPlugins[] = [
						'name'        => $pluginName,
						'by'          => $plugin["AuthorName"],
						'isConnected'    => boolval(false),
						'version'     => $plugin['Version'],
						'sourceNameId' => $appPlugin->sourceName,
						'logoPath'  	  => $appPlugin->pluginData['logoPath'],
						'description' => $plugin['Description'],
						'RequiresWP'  => $plugin['RequiresWP'] ? $plugin['RequiresWP'] : null,
						'RequiresPHP' => $plugin['RequiresPHP'] ? $plugin['RequiresPHP'] : null,
                        'activated' => false
					];
				}
			}
		}
		return $supportedOnlyInstalledPlugins;
	}

	/*
		send Installed ONLY integrations data
	*/

	static function sendInstalledOnlyIntegrationsData()
	{
		$installedOnlyIntegrations = self::getSupportedInstalledOnlyIntegrations();
		return self::createPlugins($installedOnlyIntegrations);
	}


	/*
	 this function is responsible for sending the data of the integrations (BOTH active and none active) to the API
	 it's getting the active integrations from the app instance
	 and getting the data of the supported plugins only (can be something that never been activated)
	 then it's sending the data of the active integrations to the API
	 used for full plugins sync (on activate of send plugin for example)
	*/
	static function sendCreateIntegrationsData()
	{
		$activeIntegrations = self::getAllSupportedActiveIntegrations();
		$installedOnlyIntegrations = self::getSupportedInstalledOnlyIntegrations();
		$payload = array_merge($activeIntegrations, $installedOnlyIntegrations);
		return self::createPlugins($payload);
	}


	static function createPlugins($payload) {
		$data = ['siteUrl' => CURRENT_HOST, 'plugins' => $payload];
        $client = new Client();
        return $client->make_request('POST', API_ENDPOINT . '/plugins/create-plugins', $data);
	}

	/*
	this function access the form data from the supported plugins and send it to the API
	for each supported plugin it's getting the data of the forms (CF7, Elementor, WPForms)
	the data is both the form fields and the forms locations
	this is done for full sync of the forms (for example on send plugin activation or any page save / delete)
	 */
	static function sendFormsData() {
		$res = null;

		$cf7Payload = Cf7::getAllData();
		$elementorFormsPayload = ElementorForms::getAllData();
		$wpFormsPayload = WpForms::getAllData();
		$payload = [$cf7Payload, $wpFormsPayload, $elementorFormsPayload];
		foreach ($payload as $forms) {
			$data = ['siteUrl' => CURRENT_HOST, 'forms' => $forms];
            $client = new Client();
            $response = $client->make_request('POST', API_ENDPOINT . '/forms', $data);
			if (is_wp_error($response)) {
				// TODO: handle error with log and exception
				// error_log("Error sending forms data: " . $response->get_error_message());
			}
			$res = $data;
		}
		return $res;
	}


    static function deletePlugin($sourceName) {
        $data = ['siteUrl' => CURRENT_HOST];
        $client = new Client();

        return $client->make_request('DELETE', API_ENDPOINT . "/plugins/{$sourceName}", wp_json_encode($data));
    }
	static function getPlugins() {
        $client = new Client();
        $response = $client->make_request('GET', API_ENDPOINT . '/plugins/get-plugins?siteUrl=' . CURRENT_HOST, []);
		if (is_wp_error($response)) {
//			error_log("Error fetching plugins: " . $response->get_error_message());
			return [];
		}
        if ($response->body === 'null' || $response->body === 'Unauthorized' ) {
            return [];
        }

		return $response;
	}


    static function getStatusBySourceId() {
        // foreach plugin get it's installed status and activated status - be based on the supported source names
        $sourceNames = self::$supportedPlugins;
        $plugins = get_plugins();
        $status = [];
        foreach ($sourceNames as $sourceName => $sourceId) {
            $status[$sourceId]['installed'] = false;
            // store the plugin activation status
            $status[$sourceId]['activated'] = false;
            foreach ($plugins as $plugin) {
                if ($plugin['Name'] === $sourceName) {
                    $status[$sourceId]['installed'] = true;
                    $status[$sourceId]['activated'] = self::getPluginFromAppState($sourceId)->active;
                    break;
                }
            }
        }
        return $status;
    }

    public static function activate($data) {
        $appStatePlugin = self::getPluginFromAppState($data['sourceNameId']);
        if ( ! isset($appStatePlugin)) {
            return status_header(404);
        }
        // activates the wordpress plugin using the app state value
        $result = activate_plugins($appStatePlugin->pluginFile);
        if (is_wp_error($result)) {
            return status_header(404);
        }
        return ["status" => "success"];
    }
}
