<?php
namespace Send\Classes\Integrations\WooCommerce\Components;

use ReflectionMethod;
use Send\Classes\Integrations\WooCommerce\Sync_Base;
use Send\Classes\Traits\Woo;
use WP_Error;

class WooOrders extends Sync_Base {

	const ENTITY = 'order';

	const SYNC_ACTION = 'ORDER_SYNC';
	const UPDATE_ACTION = 'ORDER_UPDATED';
	const NON_USER_COOKIE_NAME = 'emp_non_user_purchase';

	const LAST_REMOVED_ITEM_SESSION_KEY = 'emp_lastRemovedItem';

	const CART_ID_META_KEY = '_e-send_cart_id';

	const CART_ID_TTL = 3600;
	const CART_ID_SESSION_KEY = 'customCartId';
	const CART_ID_TIMESTAMP_SESSION_KEY = 'cartIdTimestamp';

	protected function get_async_update_refund_hook_name(): string {
		return static::ASYNC_UPDATE_HOOK_PREFIX . 'refund/' . static::ENTITY;
	}

	protected function register_hooks() {
		parent::register_hooks();
		add_action( 'emp/send/async/update/refund/' . static::ENTITY, [ $this, 'async_update_refund' ], 10, 4 );

		// Order related hooks:
		add_action( 'woocommerce_order_status_completed', [ $this, 'on_order_complete' ], 10, 3 );
		add_action( 'woocommerce_order_status_failed', [ $this, 'on_payment_failed' ], 10, 3 );
		add_action( 'woocommerce_checkout_order_processed', [ $this, 'trigger_checkout_order_processed' ], 10, 3 );
		add_action( 'woocommerce_store_api_checkout_order_processed', [ $this, 'trigger_api_checkout_order_processed' ], 10, 1 );
		add_action( 'woocommerce_new_order', [ $this, 'triggerNewOrder' ], 10, 3 );
		add_action( 'woocommerce_order_refunded', [ $this, 'on_refund_order' ], 10, 2 );
		add_action( 'woocommerce_update_order', [ $this, 'on_order_update' ], 10, 2 );
		add_action( 'woocommerce_add_to_cart', [ $this, 'onItemsAddedToCart' ], 10, 6 );
		add_action( 'woocommerce_cart_emptied', [ $this, 'on_cart_emptied' ], 10, 1 );

		// TODO: finish cleanup
		$orderHooks = [
			'woocommerce_remove_cart_item' => 'beforeRemovedFromCart', // Remove from Cart - easy // TODO
			'woocommerce_cart_item_removed' => 'onRemoveFromCart', // TODO
		];

		foreach ($orderHooks as $hook => $function) {
			// Get the number of parameters accepted by the function
			$reflection = new ReflectionMethod($this, $function);
			$numParams = $reflection->getNumberOfParameters();

			// Add action with dynamic number of accepted arguments
			add_action( $hook, [ $this, $function ], 10, $numParams );
		}
	}

	/**
	 * Triggered by the action scheduler and sends the refund related data:
	 *
	 * @param int $order_id
	 * @param int $refund_id
	 * @param string $trigger
	 * @param string $action
	 *
	 * @return mixed|WP_Error
	 */
	public function async_update_refund( int $order_id, int $refund_id, string $trigger, string $action ) {
		$refund = wc_get_order( $refund_id );
		$data = [
			'original_order' => $this->get_sync_data( $order_id ),
			'refund_id' => $refund_id,
			'refund_data' => $refund->get_data()
		];

		return Woo::sendMetaData( $data, $trigger,  $action );
	}

	/**
	 * @param $entity_object
	 *
	 * @return array
	 */
	protected function get_sync_data( $entity_object ): array {
		if ( is_numeric( $entity_object ) ) {
			$entity_object = wc_get_order( $entity_object );
			if ( empty( $entity_object ) ) {
				return [];
			}
		}
		$orderDate = $entity_object->get_date_created();
		$orderId = $entity_object->get_id();

		$customer = $this->get_customer_data_by_order( $entity_object );
		$cart_id = $entity_object->get_meta( self::CART_ID_META_KEY );

		$orderDetails = [
			'order_id' => $orderId,
			'type' => 'sync',
			'status' => $entity_object->get_status(),
			'total_price' => $entity_object->get_total(),
			'order_date_created' => $orderDate->getTimestamp(),
			'order_date_paid' => $entity_object->get_date_paid() ? $entity_object->get_date_paid()->getTimestamp() : 0,
			'order_date_completed' => $entity_object->get_date_completed() ? $entity_object->get_date_completed()->getTimestamp() : 0,
			'order_date_modified' => $entity_object->get_date_modified() ? $entity_object->get_date_modified()->getTimestamp() :0,
			'is_woo_customer' => true,
			'customer' => $customer,
			'payment_method' => $entity_object->get_payment_method(),
			'payment_method_title' => $entity_object->get_payment_method_title(),
			'customer_note' => $entity_object->get_customer_note(),
			'order_items' => [], // Array of order items details
			'user_data' => $customer, // same as customer info - just for consistency with other functions
			'quantity' => $entity_object->get_item_count(),
			'currency' => $entity_object->get_currency(),
			'created_via' => $entity_object->get_created_via(),
			// Add more details as needed using order object methods
		];

		// Add only if it's there:
		if ( ! empty( $cart_id ) ) {
			$orderDetails['cart_id'] = $cart_id;
		}

		// Loop through order items and get details
		foreach ( $entity_object->get_items() as $item_id => $item ) {
			$orderDetails['order_items'][] = [
				'product_id' => $item->get_product_id(),
				'variation_id' => $item->get_variation_id(),
				'name' => $item->get_name(),
				'quantity' => $item->get_quantity(),
				'subtotal' => $item->get_subtotal(),
				'total' => $item->get_total(),
				'tax_class' => $item->get_tax_class(),
				// Add more item details as needed using item object methods
			];
		}

		return $orderDetails;
	}

	protected function get_customer_data_by_order( $order ): array {
		$customer_id = $order->get_customer_id();

		$customer = [
			'billing_info' => [
				'email' => $order->get_billing_email(),
				'phone' => $order->get_billing_phone(),
				'first_name' => $order->get_billing_first_name(),
				'last_name' => $order->get_billing_last_name(),
				'zip' => $order->get_billing_postcode(),
				'address_1' => $order->get_billing_address_1(),
				'address_2' => $order->get_billing_address_2(),
				'city' => $order->get_billing_city(),
				'state' => $order->get_billing_state(),
				'country' => $order->get_billing_country(),
			],
			'shipping_info' => [
				'phone' => $order->get_shipping_phone(),
				'first_name' => $order->get_shipping_first_name(),
				'last_name' => $order->get_shipping_last_name(),
				'zip' => $order->get_shipping_postcode(),
				'address_1' => $order->get_shipping_address_1(),
				'address_2' => $order->get_shipping_address_2(),
				'city' => $order->get_shipping_city(),
				'state' => $order->get_shipping_state(),
				'country' => $order->get_shipping_country(),
			],
			'user' => null,
		];

		// 0 means guest:
		if ( 0 < $customer_id ) {
			$user = get_user_by( 'ID', $customer_id );
			if ( $user ) {
				$customer['user'] = $this->get_user_data( $user );
			}
		}

		return $this->sanitize_customer_info( $customer );
	}

	private function sanitize_customer_info( array $customer_info ): array {
		$customer_parts = [
			'user',
			'billing_info',
			'shipping_info',
		];

		foreach ( $customer_parts as $part ) {
			if ( ! is_array( $customer_info[ $part ] ) ) {
				continue;
			}
			$set_to_null = true;
			foreach ( $customer_info[ $part ] as $key => $value ) {
				if ( ! empty( $value ) ) {
					$set_to_null = false;
					break;
				}
			}
			if ( $set_to_null ) {
				$customer_info[ $part ] = null;
			}
		}

		return $customer_info;
	}

	// action hooks below...

	public function beforeRemovedFromCart($cartItem, $cart) {
		$removedItem = $cart->get_cart_item($cartItem);
		$item_data = [
			'id' => $removedItem['product_id'],
			'quantity' => $removedItem['quantity'],
			'total' => $removedItem['line_total']
		];
		WC()->session->set( self::LAST_REMOVED_ITEM_SESSION_KEY, $item_data );
	}

	public function onRemoveFromCart($removedCartItemId, $cart) {
		$removedItem = WC()->session->get( self::LAST_REMOVED_ITEM_SESSION_KEY );
		// Get the total quantity of items in the cart
		if ( empty( $removedItem ) ) {
			error_log('Fatal error - Woo Commerce Logger: No item was removed from the cart');
			return;
		}
        $cart_id = self::createOrGetCartId();
        $cartLink = wc_get_cart_url();
		// get the removed item from the cart
		$removedItemQuantity = $removedItem['quantity'];
		$valueRemoved = $removedItem['total'];
		// get the cart value
		$newCartValue = $cart->get_cart_contents_total() - $valueRemoved;
		$newQuantityInCart = $cart->get_cart_contents_count();
		$currency = get_woocommerce_currency();

		if ( $newCartValue < 0 ) {
			$newCartValue = 0;
		}
		$removedItemId = $removedItem['id'];
		// get the product by the id
		$product = wc_get_product($removedItemId);
		$productData = $product->get_data();
		// get the price of the product
		$productPrice = $product->get_price();
		$totalItemsInCart = WC()->cart->get_cart_contents_count();
		$current_user = \wp_get_current_user();
		if ( 0 !== $current_user->ID ) {
			$userInfo = $this->get_user_data( $current_user ) ;
		} else {
			$userInfo = $this->extract_customer_from_cookie();
		}
		// Check if the cart is empty
		$hasUserDetails = ! empty( $userInfo );
		if ( 0 === $totalItemsInCart ) {
			// Log a message when the cart is completely emptied
			$this->onCartEmpty( $removedCartItemId, $cart, $cart_id, $hasUserDetails, $userInfo );
		} else {
			Woo::sendMetaData( [
				'cart_item_key' => $removedCartItemId,
				'has_user_details' => $hasUserDetails,
				'user' => $userInfo,
				'cart_id' => $cart_id,
				'value_removed' => $valueRemoved,
				'single_product_price' => $productPrice,
				'quantity_in_cart' => $newQuantityInCart,
				'quantity_removed' => $removedItemQuantity,
				'product_id_removed' => $removedItemId,
				'customer' => $userInfo,
				'product_data' => $productData,
                'cart_link' => $cartLink,
				'currency' => $currency,
			], 'remove from cart', 'ON_REMOVAL_FROM_CART');
		}
	}

	public function on_cart_emptied( $persistent_cart ) {
		$this->reset_cart_id();
		//TODO: should send an event?
	}

	protected function get_user_data( $user ): array {
		return [
			'user_id' => $user->ID,
			'email' => $user->user_email,
			'first_name' => $user->first_name,
			'last_name'  => $user->last_name,
			'registered' => $user->user_registered,
            'roles' => $user->roles,
		];
	}

	public function onCartEmpty( $removedCartItemId, $cart, $cart_id, $hasUserDetails = false, $userInfo = [] ) {
		error_log('Cart onCartEmpty Detected');
		$userInfo = $this->extract_customer_from_cookie();
		$currency = get_woocommerce_currency();
		Woo::sendMetaData( [
			'cart_item_key' => $removedCartItemId,
			'has_user_details' => $hasUserDetails,
			'customer' => $userInfo,
			'total_amount' => 0,
			'total_quantity' => 0,
			'currency' => $currency,
			'cart_id' => $cart_id,
			], 'cart empty', 'ON_CART_EMPTY' );
		// make the cart cookie empty
		WC()->session->set( self::CART_ID_TIMESTAMP_SESSION_KEY, time() - self::CART_ID_TTL );
	}

	public function trigger_checkout_order_processed( $order_id, $posted_data, $order ) {
		$this->triggerNewOrder( $order_id, $order );
	}

	public function trigger_api_checkout_order_processed( $order ) {
		$this->triggerNewOrder( $order->get_id(), $order );
	}

	/**
	 * trigger a single new order sync.
	 * IMPORTANT: cannot use this method for bulk updates.
	 *
	 * @param $orderId
	 * @param $order
	 *
	 * @return void
	 */
	public function triggerNewOrder( $orderId, $order ) {
		// once per request is enough.
		static $already_triggered = false;

		if ( $already_triggered ) {
			return;
		}
	 	$nonUserDetails = $this->get_customer_data_by_order( $order );

		// Encode the array as a JSON string
		$encodedDetails = wp_json_encode( $nonUserDetails );
		$this->onOrderAmount( $order );
		if ( ! empty( $_COOKIE[ self::NON_USER_COOKIE_NAME ] ) ) {
			// Proper way to remove a cookie
			setcookie( self::NON_USER_COOKIE_NAME, $encodedDetails, time() + ( 10 * 365 * 24 * 60 * 60 ), '/' );
			error_log('Woo Commerce Logger: User Purchase Details Updated');
		} else {
			error_log('Woo Commerce Logger: New User Purchase Details Cookie Created');
		}

		// Set the cookie with the JSON string
		setcookie( self::NON_USER_COOKIE_NAME, $encodedDetails, time() + 3600, '/' );
		$cart_id = self::createOrGetCartId();
		$order->update_meta_data( self::CART_ID_META_KEY, $cart_id );
		$order->save();
		$this->onNewOrder( $order, $encodedDetails );
		$already_triggered = true;
	}

    public function on_order_update( $orderId, $order ) {
	    $this->add_scheduled_action( $this->get_async_update_hook_name(), [
		    'id' => $orderId,
		    'trigger' => 'order updated',
		    'action' => 'ORDER_UPDATED',
	    ], static::ASYNC_UPDATE_GROUP );
   }

   // TODO: do we need this method? maybe the decision if its a first order should be done on the app's side?
    public function onNewOrder( $order, $nonUserDetails ) {

	    $trigger = 'guest user has ordered';

	    if ( $order && $order->get_customer_id() ) {
		    // Get all orders for the user, excluding the current order
		    $customerOrders = wc_get_orders( [
			    'customer_id' => $order->get_customer_id(),
			    'exclude' => [ $order->get_id() ],
		    ] );
		    $trigger = empty( $customerOrders ) ? 'first order' : 'Subsequent Order Detected';
	    }

	    $this->add_scheduled_action( $this->get_async_update_hook_name(), [
		    'id' => $order->get_id(),
		    'trigger' => $trigger,
		    'action' => 'NEW_ORDER',
	    ], static::ASYNC_UPDATE_GROUP );
   }

    public function on_refund_order( $order_id, $refund_id ) {
	    $this->add_scheduled_action( $this->get_async_update_refund_hook_name(), [
		    'id' => $order_id,
			'refund_id' => $refund_id,
		    'trigger' => 'refund order',
		    'action' => 'REFUND_ORDER',
	    ], static::ASYNC_UPDATE_GROUP );
	}

	public static function createOrGetCartId() {
		// Check if a custom cart ID exists and if it's still valid
		$customCartId = WC()->session->get( self::CART_ID_SESSION_KEY );
		$cartIdTimestamp = WC()->session->get( self::CART_ID_TIMESTAMP_SESSION_KEY );

		if ( ! $customCartId || ! $cartIdTimestamp || ( time() - $cartIdTimestamp ) > self::CART_ID_TTL ) {
			// Generate a new custom cart ID and store it with the current timestamp
			$siteUrlHash = substr( md5( CURRENT_HOST ), 0, 8 );
			$newCartId = uniqid( 'cart_' ) . '_' . $siteUrlHash;
			WC()->session->set( self::CART_ID_SESSION_KEY, $newCartId );
			WC()->session->set( self::CART_ID_TIMESTAMP_SESSION_KEY, time() );
			return $newCartId;
		}

		// Return the existing custom cart ID
		return $customCartId;
	}

	protected function reset_cart_id() {
		if ( ! WC()->session ) {
			return;
		}

		WC()->session->set( self::CART_ID_SESSION_KEY, null );
		WC()->session->set( self::CART_ID_TIMESTAMP_SESSION_KEY, null );
	}

	private function extract_customer_from_cookie(): array {
		$customer = [
			'user' => null,
			'billing_info' => null,
			'shipping_info' => null,
		];
		if ( ! empty( $_COOKIE[ self::NON_USER_COOKIE_NAME ] ) ) {
			$customer = json_decode( stripslashes( $_COOKIE[ self::NON_USER_COOKIE_NAME ] ), true );
		}

		return $customer;
	}

	/**
	 * TODO: use action scheduler
	 *
	 * @param $cartId
	 * @param $productId
	 * @param $requestQuantity
	 * @param $variationId
	 * @param $variation
	 * @param $cartItemData
	 *
	 * @return mixed
	 */
	public function onItemsAddedToCart( $cartId, $productId, $requestQuantity, $variationId, $variation, $cartItemData ) {
		$customer = $this->extract_customer_from_cookie();
		if ( WC()->session->get( self::CART_ID_SESSION_KEY ) ) {
			// Get the timestamp when the cartId was last updated
			$cartIdTimestamp = WC()->session->get( self::CART_ID_TIMESTAMP_SESSION_KEY );
			// Check if the current time is within the TTL
			if ( ( time() - $cartIdTimestamp ) < self::CART_ID_TTL ) {
				WC()->session->set( self::CART_ID_TIMESTAMP_SESSION_KEY, time() );
			}
		}

		$cart = WC()->cart;
		$totalItemsInCart = $cart->get_cart_contents_count();
		$cartLink = wc_get_cart_url();
		$currency = get_woocommerce_currency();
		$product = wc_get_product($productId);
		$productPrice = floatval($product->get_price()); // Ensure productPrice is a float
		$currentCartValue = floatval($cart->get_cart_contents_total()); // Ensure currentCartValue is a float
		$newCartValueInt = $currentCartValue + $productPrice; // Add the values correctly
		$newCartValue = strval($newCartValueInt); // Convert the new cart value back to a string
		$productData = $product->get_data();
		$cartId = self::createOrGetCartId();

		$cartData = [
			'product_id' => $productId,
			'quantity' => $requestQuantity,
			'product_price' => $productPrice,
			'total_added_price' => $requestQuantity * $productPrice,
			'variation_id' => $variationId,
			'variation' => $variation,
			'cart_item_data' => $cartItemData,
			'product_data' => $productData,
		];
		return Woo::sendMetaData( [
			'cart' => $cartData,
		    'cart_link' => $cartLink,
		    'total_items_in_cart' => $totalItemsInCart,
		    'cart_id' => $cartId,
		    'customer' => $customer,
		    'currency' => $currency,
		    'cart_value' => $newCartValue,
			], 'add to cart', 'ADD_ITEM_TO_CART' );
	}

	// Corresponding functions
	public function on_order_complete( $order_id, $order, $status_transition ) {
		$this->add_scheduled_action( $this->get_async_update_hook_name(), [
			'id' => $order_id,
			'trigger' => 'order completed',
			'action' => 'USER_PURCHASE',
		], static::ASYNC_UPDATE_GROUP );

		$this->reset_cart_id();
	}

	// TODO: MAKE IT WORK. FOR NOW IT'S OK THAT IT'S NOT WORKING
	public function onShippingCompleted($orderId) {
		return Woo::sendMetaData(['order_id' => $orderId], 'order shipped', 'orderShipped');
	}

	// TODO: MAKE IT WORK. FOR NOW IT'S OK THAT IT'S NOT WORKING
	public function on_payment_failed( $order_id, $order, $status) {
		error_log( "Payment Failed: Order ID " . $order_id );
		$this->add_scheduled_action( $this->get_async_update_hook_name(), [
			'id' => $order_id,
			'trigger' => 'payment failed',
			'action' => 'PAYMENT_FAILED',
		], static::ASYNC_UPDATE_GROUP );
	}

	public function onInvoiceSent($orderId) {
		return Woo::sendMetaData(['order_id' => $orderId], 'invoice sent', 'invoiceSent');
	}

	// TODO: MAKE IT WORK. FOR NOW IT'S OK THAT IT'S NOT WORKING
	public function orderShipped($orderId) {
		return Woo::sendMetaData(['order_id' => $orderId], 'order shipped', 'orderShipped');
	}

	// TODO: MAKE IT WORK. FOR NOW IT'S OK THAT IT'S NOT WORKING

	public function onOrderAmount( $order ) {
		$orderAmount = $order->get_total();
		$currency = $order->get_currency();
		return Woo::sendMetaData(['order_id' => $order->get_id(), 'order_amount' => $orderAmount, 'currency' => $currency], 'order amount', 'ORDER_AMOUNT');
	}

	protected function get_next_page( bool $is_first ) {
		$orders = wc_get_orders(
			[
				'limit' => $this->get_limit(),
				'page' => $this->get_current_page(),
				'paginate' => true,
				'type' => 'shop_order' // what about refunds?
			]
		);

		if ( $is_first ) {
			$this->set_max_pages( $orders->max_num_pages );
			$this->set_total_items( $orders->total );
		}

		return ! empty( $orders ) ? $orders->orders : new WP_Error( static::GET_NEXT_PAGE_ERROR_CODE, 'no orders found' );
	}

	protected function process( array $items ) {
		// Loop through each order and get full details
		$orders_data = [];
		foreach ( $items as $item ) {
			/** @var \WC_Order $item */
			$orders_data[] = $this->get_sync_data( $item );
		}

		return parent::process( $orders_data );
	}
}
