<?php
namespace Send\Classes\Integrations\WooCommerce\Components;

use ReflectionMethod;
use Send\Classes\Integrations\WooCommerce\Sync_Base;
use Send\Classes\Traits\Woo;

class WooProducts extends Sync_Base {

	const ENTITY = 'product';

	const SYNC_ACTION = 'PRODUCTS_SYNC';
	const UPDATE_ACTION = 'PRODUCT_UPDATED';

	protected function get_next_page( bool $is_first = false ) {
		$args = [
			'limit' => $this->get_limit(),
			'page' => $this->get_current_page(),
			'paginate' => true,
		];

		$products = wc_get_products( $args );

		if ( $is_first ) {
			$this->set_max_pages( $products->max_num_pages );
			$this->set_total_items( $products->total );
		}

		return ! empty( $products ) ? $products->products : new \WP_Error( static::GET_NEXT_PAGE_ERROR_CODE, 'No products found' );
	}

	/**
	 * @param $entity_object
	 *
	 * @return array
	 */
	protected function get_sync_data( $entity_object ): array {
		if ( is_numeric( $entity_object ) ) {
			$entity_object = wc_get_product( $entity_object );
			if ( empty( $entity_object ) ) {
				return [];
			}
		}
		$data = $entity_object->get_data();
		$data['add_to_cart_url'] = $entity_object->add_to_cart_url();
		$data['permalink'] = $entity_object->get_permalink();
		$image_id =  $entity_object->get_image_id();
		if ( $image_id ) {
			$image_src = wp_get_attachment_image_src( $image_id, 'thumbnail' );
			if ( ! empty( $image_src[0] ) ) {
				$data[ 'image_src'] = $image_src[0];
			}
		}
		return $data;
	}

	protected function process( array $items ) {
		$products_data = [];
		foreach ( $items as $item ) {
			$products_data[] = $this->get_sync_data( $item );
		}
		parent::process( $products_data );
	}

		// Define a function to retrieve all refunds

	public static function getAllCategories() {
		return get_terms( array(
			'taxonomy'   => 'product_cat', // Taxonomy for product categories
			'hide_empty' => false, // Set to false to include empty categories
		) );
	}

	public static function getAllCoupons() {
		$coupons = get_posts( array(
			'post_type'   => 'shop_coupon', // Type of post representing coupons
			'numberposts' => -1, // Retrieve all coupons (-1 for all)
		) );
		$couponsPayload = [];
		foreach ($coupons as $coupon) {
			$couponData      = get_post_meta($coupon->ID);
			$coupnsPayload[] = [
				"meta" => $couponData,
				"data" => $coupon,
			];
		}
		error_log('Woo Commerce Logger: All Coupons Sync started ');
		return $coupnsPayload;
	}

	protected function register_hooks() {
		parent::register_hooks();
		// Product-related hooks
		$orderHooks = [
			'transition_post_status'          => 'onProductStatusChange',
			'created_term'                    => 'onCreateCategory',
			'delete_term'                     => 'onDeleteCategory',
			'edited_term'                     => 'onEditCategory',
			'woocommerce_product_set_stock_status' => 'onProductStockStatusChange',
			'woocommerce_update_product' => 'onProductUpdated',
			//'woocommerce_coupon_options_save' => 'onCouponSave', //TODO: move to async
			//'comment_post'                    => 'onProductReview', //TODO: move to async
            //'update_option_woocommerce_currency' => 'onUpdateCurrency', //TODO: move to async
		];

		foreach ($orderHooks as $hook => $function) {
			// Get the number of parameters accepted by the function
			$reflection = new ReflectionMethod( $this, $function) ;
			$numParams  = $reflection->getNumberOfParameters();
			// Add action with dynamic number of accepted arguments
			add_action( $hook, [ $this, $function ], 10, $numParams );
		}
		// tried 'delete_post' and 'delete;_shop_coupon' / etc - no luck
	}

	// action hooks below...

 	public function onProductUpdated( $product_id ) {
		$this->add_scheduled_action( $this->get_async_update_hook_name(), [
				'id' => $product_id,
				'trigger' => 'update product',
				'action' => static::UPDATE_ACTION,
			], static::ASYNC_UPDATE_GROUP );
	}

	public function onProductStockStatusChange( $product_id, $stockStatus, $product ) {
		switch ( $stockStatus ) {
			case 'outofstock':
				$trigger = 'product out of stock';
				break;
			case 'instock':
				$trigger = 'product in stock';
				break;
			default:
				$trigger = 'product stock status changed [' . $stockStatus . ']';
				break;
		}
		$this->add_scheduled_action( $this->get_async_update_hook_name(), [
			'id' => $product_id,
			'trigger'    => $trigger,
			'action'     => static::UPDATE_ACTION,
		], static::ASYNC_UPDATE_GROUP );
	}

	/**
	 * TODO: the trigger is wrong.
	 *
	 * @param $newStatus
	 * @param $oldStatus
	 * @param $post
	 *
	 * @return void
	 */
	public function onProductStatusChange($newStatus, $oldStatus, $post) {
		// workaround - check issue in github
		// https://github.com/woocommerce/woocommerce/issues/23610
		if ( empty( wc_get_product( $post->ID ) ) ) {
			return;
		}

		$trigger = 'product status changed [`' . $oldStatus . '` -> `' . $newStatus . '`]';

		$this->add_scheduled_action( $this->get_async_update_hook_name(), [
			'id' => $post->ID,
			'trigger'    => $trigger,
			'action'     => static::UPDATE_ACTION,
		], static::ASYNC_UPDATE_GROUP );
	}

	// TODO: move to async
	public function onUpdateCurrency($currency) {
		error_log('Woo Commerce Logger: Currency Updated to ' . $currency);
		return Woo::updateSiteCurrency();
	}

	// TODO: move to async
	public function onProductReview($commentId, $commentApproved) {
		$comment = get_comment( $commentId );
		return Woo::sendMetaData($comment, 'product review', 'NEW_COMMENT');
	}

	// TODO: move to async
	public function onCouponSave($id) {
		error_log('Woo Commerce Logger: Coupon ID: ' . $id);
		$coupon = new \WC_Coupon( $id );
		$couponData = $coupon->get_data();
		return Woo::sendMetaData($couponData, 'coupon save', 'NEW_COUPON');
	}

	// TODO: move to async
	public function onCouponDelete($id) {
		// Not working yet.
		error_log('Woo Commerce Logger: Deleted Coupon ID: ' . $id);
		return Woo::sendMetaData(['id' => $id], 'coupon delete', 'couponDelete');
	}

	// TODO: move to async
	public function onCreateCategory($termId, $ttId, $taxonomy) {
		if ($taxonomy === 'product_cat') {
			$category = get_term( $termId, 'product_cat' ); // Replace 'product_cat' with the appropriate taxonomy if needed
			if ( ! is_wp_error( $category ) ) {
				error_log('Woo Commerce Logger: Category Created!');
				Woo::sendMetaData($category, 'new category', 'CATEGORY_CREATED');
			}
		}
	}

	// TODO: move to async
	public function onDeleteCategory($termId, $ttId, $taxonomy) {
		if ($taxonomy === 'product_cat') {
			$category = get_term( $termId, 'product_cat' ); // Replace 'product_cat' with the appropriate taxonomy if needed
			if ( ! is_wp_error( $category ) ) {
				error_log('Woo Commerce Logger: Category Deleted');
				return Woo::sendMetaData($category, 'delete category', 'CATEGORY_DELETED');
			}
		}
	}

	// TODO: move to async
	public function onEditCategory($termId, $ttId, $taxonomy) {
		if ( $taxonomy === 'product_cat' ) {
			$category = get_term( $termId, 'product_cat' ); // Replace 'product_cat' with the appropriate taxonomy if needed
			if ( ! is_wp_error( $category ) ) {
				error_log('Woo Commerce Logger: Category Edited');
				return Woo::sendMetaData($category, 'edit category', 'CATEGORY_UPDATED');
			}
		}
	}

}
