<?php

namespace Send\Classes\Integrations\WooCommerce;

use Send\Classes\Traits\Woo;
use Send\Classes\Logger\Logger;
use WP_Error;
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * class Sync_Base
 **/
abstract class Sync_Base {
	// TODO: what is the relevant initial value?
	protected int $limit = 100;

	protected int $current_page = 0;

	protected int $max_pages = -1;

	protected int $total_items = -1;

	const OPTION_KEY_PREFIX_CURRENT_PAGE = 'send_sync_current_page_';
	const OPTION_KEY_PREFIX_PROGRESS_PERCENT = 'send_sync_progress_percent_';
	const OPTION_KEY_PREFIX_TOTAL = 'send_sync_total_';
	const OPTION_KEY_PREFIX_MAX_PAGES = 'send_sync_max_pages_';

	const SYNC_HOOK_PREFIX = 'send_sync_';
	const ASYNC_UPDATE_HOOK_PREFIX = 'emp/send/async/update/';

	const ENTITY = '';

	const SYNC_GROUP = 'send_initial_sync';
	const ASYNC_UPDATE_GROUP = 'send_async_update';

	const GET_NEXT_PAGE_ERROR_CODE = 'get_next_page_error';
	const ASYNC_UPDATE_ERROR_CODE = 'async_update_error';

	const SYNC_ACTION = '';
	const UPDATE_ACTION = '';

	public function get_limit(): int {
		return $this->limit;
	}

	public function set_limit( int $limit ) {
		$this->limit = $limit;
	}

	public function get_current_page(): int {
		return $this->current_page;
	}

	public function set_current_page( int $current ) {
		$this->current_page = $current;
	}

	public function get_max_pages(): int {
		return $this->max_pages;
	}

	public function set_max_pages( int $max_pages ) {
		$this->max_pages = $max_pages;
	}

	public function get_total_items(): int {
		return $this->total_items;
	}

	public function set_total_items( int $total_items ) {
		$this->total_items = $total_items;
	}

	protected function get_progress_data() {
		// TODO: add prefix of environment to avoid conflicts
		$current_page = get_option( static::OPTION_KEY_PREFIX_CURRENT_PAGE . static::ENTITY, 1 );
		$this->set_current_page( intval( $current_page ) );
	}

	protected function update_sync_data() {
		update_option( static::OPTION_KEY_PREFIX_TOTAL . static::ENTITY, $this->get_total_items() );
		update_option( static::OPTION_KEY_PREFIX_MAX_PAGES . static::ENTITY, $this->get_max_pages() );
	}

	protected function update_progress_data() {
		$progress = 0 < $this->get_max_pages() ? intval( ( $this->get_current_page() / $this->get_max_pages() ) * 100 ) : 1;
		update_option( static::OPTION_KEY_PREFIX_CURRENT_PAGE . static::ENTITY, $this->get_current_page() );
		update_option( static::OPTION_KEY_PREFIX_PROGRESS_PERCENT . static::ENTITY, $progress );
	}

	protected function cleanup() {
		$options = $this->get_option_names_prefixes();

		foreach ( $options as $option ) {
			delete_option( $option . static::ENTITY );
		}
	}
	
	public function register_sync( bool $first = false ): bool {
		return 0 !== as_schedule_single_action( time() + 3, $this->get_sync_hook_name(), [ 'first' => $first ], static::SYNC_GROUP );
	}

	public function fetch_and_sync( bool $first ) {
		// refresh current-page:
		$option_value = get_option( static::OPTION_KEY_PREFIX_CURRENT_PAGE . static::ENTITY, $this->current_page );
		$this->set_current_page( intval( $option_value ) + 1 );

		if ( ! $first && ( $this->get_max_pages() < $this->get_current_page() ) ) {
			// delete temp data & bail:
			$this->cleanup();
			return true;
		}

		// run the data query:
		$entity_items = $this->get_next_page( $first );

		if ( is_wp_error( $entity_items ) ) {
			return $this->report_error( $entity_items );
		}

		if ( $first ) {
			// save max-pages & total-items:
			$this->update_sync_data();
		}
		// save the current page & progress %:
		$this->update_progress_data();

		// format & send:
		$this->process( $entity_items );
		$this->register_sync();
		return true;
	}

	protected function get_sync_hook_name(): string {
		return static::SYNC_HOOK_PREFIX . static::ENTITY;
	}

	protected function get_async_update_hook_name(): string {
		return static::ASYNC_UPDATE_HOOK_PREFIX . static::ENTITY;
	}

	protected function report_error( \WP_Error $error, $trigger = 'initial_sync' ): WP_Error {
		// TODO: replace with a proper logger:
		error_log( sprintf( '%1$s | code: %2$s | message: %3$s', static::ENTITY, $error->get_error_code(), $error->get_error_message() ) );
		$error_array = [
			'code' => $error->get_error_code(),
			'reason' => $error->get_error_message(),
			'data' => $error->get_error_data(),
		];
		if ( 'initial_sync' ===  $trigger ) {
			Woo::sendInitialSyncData( [
					'error' => $error_array,
					'chunkLimit' => $this->get_limit(),
					'totalChunks' => $this->get_max_pages(),
					'totalItems' => $this->get_total_items(),
					'currentChunk' => $this->get_current_page(),
				], 'initial sync error', static::ENTITY );
		} else {
			Woo::sendMetaData( [
				'error' => $error_array,
			], $trigger,  static::UPDATE_ACTION . '_ERROR' );
		}

		return $error;
	}

	protected function add_scheduled_action( $hook, $args, $group, $time = null ) {
		as_schedule_single_action( $time ?? time() + 1, $hook, $args, $group );
	}

	/**
	 * calc_memory_chunck
	 *
	 * dynamically calculate average entity size.
	 * can be used to define the query `limit` arg.
	 *
	 * @return int|null
	 */
	protected function calc_memory_chunk(): ?int {
		$max_memory_per_chunk = 300 * 1024 * 1024; // 300MB in bytes
		$sample_size = 10; // Number of products to sample

		// save initial values:
		$limit = $this->get_limit();
		$current_page = $this->get_current_page();
		$max_pages = $this->get_max_pages();
		$total_items = $this->get_total_items();

		// align to sample size:
		$this->set_limit( $sample_size );
		$this->set_current_page( 1 );
		$this->set_max_pages( 1 );

		// Fetch a small sample of products to estimate average size
		$memory_usage_before = memory_get_usage();

		$this->get_next_page( true );

		$memory_usage_after = memory_get_usage();

		// restore initial values:
		$this->set_limit( $limit );
		$this->set_current_page( $current_page );
		$this->set_max_pages( $max_pages );
		$this->set_total_items( $total_items );
		$this->cleanup(); // todo: is required?

		if ( empty( $sample ) ) {
			return null;
		}

		// Calculate the memory usage of the sample
		$sample_size = count( $sample );

		// Estimate average size per entity
		$average_size = ( $memory_usage_after - $memory_usage_before ) / $sample_size;

		// Calculate the chunk size based on the 50MB memory limit
		return floor( $max_memory_per_chunk / $average_size );
	}

	protected function register_hooks() {
		add_action( $this->get_sync_hook_name(), [ $this, 'fetch_and_sync' ], 10, 1 );
		add_action( $this->get_async_update_hook_name(), [ $this, 'async_update' ], 10, 3 );
	}

	/**
	 * @param bool $is_first
	 *
	 * @return \WP_Error|array
	 */
	abstract protected function get_next_page( bool $is_first );


	/**
	 * @param mixed|int $entity_object either the actual object or its numeric id
	 *
	 * @return array
	 */
	abstract protected function get_sync_data( $entity_object ): array;

	public function async_update( int $id, string $trigger, string $action ) {
		$data = $this->get_sync_data( $id );
		if ( empty( $data ) ) {
			$error = new WP_Error( static::ASYNC_UPDATE_ERROR_CODE, static::ENTITY . ' not found. id: ' . $id, [
				'trigger' => $trigger,
				'action' => $action,
			] );
			return $this->report_error( $error, $trigger );
		}
		return Woo::sendMetaData( $data, $trigger,  $action );
	}

	protected function process( array $items ) {
		$chunkLimit = $this->get_limit();
		$totalChunks = $this->get_max_pages();
		$totalItems = $this->get_total_items();
		$currentChunk = $this->get_current_page();
		Logger::log('Processing ' . static::ENTITY, 'debug', [ 'entity' => static::ENTITY, 'chunkLimit' => $chunkLimit, 'totalChunks' => $totalChunks, 'totalItems' => $totalItems, 'currentChunk' => $currentChunk, 'siteUrl' => strval(CURRENT_HOST)]);

		return Woo::sendInitialSyncData( [
			'items' => json_encode( $items ),
			'chunkLimit' => $chunkLimit,
			'totalChunks' => $totalChunks,
			'totalItems' => $totalItems,
			'currentChunk' => $currentChunk,
		], 'sync data (activate plugin)', static::ENTITY );
	}

	protected function get_option_names_prefixes(): array {
		return [
			static::OPTION_KEY_PREFIX_CURRENT_PAGE,
			static::OPTION_KEY_PREFIX_MAX_PAGES,
			static::OPTION_KEY_PREFIX_PROGRESS_PERCENT,
			static::OPTION_KEY_PREFIX_TOTAL,
		];
	}

	protected function init() {
		$option_value = get_option( static::OPTION_KEY_PREFIX_CURRENT_PAGE . static::ENTITY, $this->current_page );
		$this->current_page = intval( $option_value );

		$option_value = get_option( static::OPTION_KEY_PREFIX_TOTAL . static::ENTITY, $this->total_items );
		$this->total_items = intval( $option_value );

		$option_value = get_option( static::OPTION_KEY_PREFIX_MAX_PAGES . static::ENTITY, $this->max_pages );
		$this->max_pages = intval( $option_value );
	}

	public function __construct() {
		$this->register_hooks();

		// force loading from options table:
		$this->init();
	}
}