<?php
/*
Plugin Name: SofiaLoger
Description: Your one stop shop for all your logging needs
Version: 0.0.1
Author: Yuval Gal - Elementor
Author URI: https://www.linkedin.com/in/yuval-gal-team-leader/
*/
namespace Send\Classes\Logger;

class Logger {
    private static function get_db() {
        global $wpdb;
        return $wpdb;
    }

    private static function create_log_table() {
        $wpdb = self::get_db();
        $table_name = $wpdb->prefix . APP_NAME.'_logs';
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE IF NOT EXISTS $table_name (
            id INT AUTO_INCREMENT PRIMARY KEY,
            logs LONGTEXT NOT NULL,
            buffer_ttl TIMESTAMP NOT NULL
        ) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    private static function initialize_log_entry() {
        $wpdb = self::get_db();
        $table_name = $wpdb->prefix . APP_NAME.'_logs';

        // Check if a row exists; if not, insert the first row
        $row_exists = $wpdb->get_var("SELECT id FROM $table_name WHERE id = 1");
        if (!$row_exists) {
            $buffer_ttl = (new \DateTime())->modify('+'.LOGGER_MAX_TTL)->format('Y-m-d H:i:s');
            $wpdb->insert(
                $table_name,
                [
                    'logs' => json_encode(['messages' => []]), // Initialize with an empty "messages" array
                    'buffer_ttl' => $buffer_ttl,
                ]
            );
        }
    }

    public static function log($message, $level = 'error', $payload = []) {
        if (!defined('LOGGER_ENDPOINT')) {
            return;
        }
        try {
            $wpdb = self::get_db();
            $table_name = $wpdb->prefix . APP_NAME . '_logs';

            // Ensure table exists
            self::create_log_table();

            // Ensure log entry is initialized
            self::initialize_log_entry();

            // Gather log details
            $log_entry = [
                'created_at' => (new \DateTime())->format('Y-m-d H:i:s'),
                'php_version' => phpversion(),
                'wp_version' => get_bloginfo('version'),
                'url' => isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : 'unknown',
                'level' => $level,
                'message' => $message,
                'payload' => $payload,
                'backtrace' => debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS)
            ];

            // Append the log entry to the existing logs
            $log_entry_json = json_encode($log_entry);
            $wpdb->query($wpdb->prepare(
                "UPDATE $table_name
            SET logs = JSON_MERGE_PATCH(logs, JSON_OBJECT('messages', JSON_ARRAY_APPEND(JSON_EXTRACT(logs, '$.messages'), '$', %s)))
            WHERE id = 1",
                $log_entry_json
            ));

            // Check buffer size and TTL
            self::checkBuffer();
        }
        catch (\Exception $e) {
            error_log('Error logging: ' . $e->getMessage());
        }
    }

    private static function checkBuffer() {
        $wpdb = self::get_db();
        $table_name = $wpdb->prefix . APP_NAME.'_logs';

        // Get the current log size and TTL
        $query = $wpdb->prepare("
            SELECT CHAR_LENGTH(logs) as total_size, logs, buffer_ttl
            FROM $table_name
            WHERE id = 1
        ");
        $row = $wpdb->get_row($query);

        if ($row && ($row->total_size > LOGGER_MAX_SIZE || $row->buffer_ttl <= date('Y-m-d H:i:s'))) {
            // Clear the logs if size exceeds max buffer or TTL has passed
            if (is_admin()) {
                self::sendLogs($row->logs);
            }
        }
    }

    private static function sendLogs($logs) {
        $wpdb = self::get_db();
        $table_name = $wpdb->prefix . APP_NAME.'_logs';

        // Fetch logs to be sent
        $logs_data = $wpdb->get_var("SELECT logs FROM $table_name WHERE id = 1");

        if ($logs_data) {
            error_log('Should process logs');
            self::request($logs);

            // Clear the logs data and set a new TTL
            $wpdb->update(
                $table_name,
                [
                    'logs' => json_encode(['messages' => []]), // Reset to an empty "messages" array
                    'buffer_ttl' => (new \DateTime())->modify('+'.LOGGER_MAX_TTL)->format('Y-m-d H:i:s')
                ],
                ['id' => 1]
            );
        }
    }

    public static function request($payload) {
        // Initialize cURL session
        $ch = curl_init();

        // Convert payload to JSON
        $jsonPayload = json_encode(['payload' => $payload]);

        // Set cURL options
        curl_setopt($ch, CURLOPT_URL, LOGGER_ENDPOINT);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonPayload); // Send JSON payload
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); // Get the response as a string
        curl_setopt($ch, CURLOPT_HEADER, 0); // Do not include the header in the output
        curl_setopt($ch, CURLOPT_TIMEOUT, 20); // Set timeout to 20 seconds
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1); // Follow redirects

        // Set non-blocking call
        curl_setopt($ch, CURLOPT_NOSIGNAL, 1); // This option is only available in PHP versions greater than 5.3.0
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, 900); // Timeout for async, adjust as needed

        // Set headers for JSON payload
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'Content-Length: ' . strlen($jsonPayload)
        ]);

        $response = curl_exec($ch);

        if ($response === false) {
            // Log the cURL error if the request failed
            error_log('cURL error: ' . curl_error($ch));
        } else {
            // Log the response if the request was successful
            error_log('Backend response: ' . $response);
        }

        curl_close($ch);
    }

}
