<?php
namespace connect;

use connect\classes\data;
use connect\classes\service;
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Class Client
 */
class Client {
	const BASE_URL = '';
	private bool $refreshed = false;

	public static ?Client $instance = null;

	/**
	 * get_instance
	 * @return Client|null
	 */
	public static function get_instance(): ?Client {
		if ( ! self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	public static function get_site_info(): array {
		return [
			// Which language to return.
			'site_lang' => get_bloginfo( 'language' ),
			// site to connect
			'site_url' => trailingslashit( home_url() ),
			// current user
			'local_id' => get_current_user_id(),
			// Webhook URL
			'webhook_url' => rest_url( 'send/v1/webhook' ),
		];
	}

	public function make_request( $method, $endpoint, $body = [], array $headers = [], $file = false, $file_name = '' ) {
        // Define the additional headers
        $additional_headers = [
            'x-elementor-send' => SEND_VERSION,
            'x-elementor-apps' => 'send',
        ];

		$site_id = data::get_client_id();

		if ( ! empty( $site_id ) ) {
			$additional_headers['site-id'] = $site_id;
		}

		// Replace the headers array
        $headers = array_replace_recursive($additional_headers, $headers);

        $headers = array_replace_recursive(
			$headers,
			$this->is_connected() ? $this->generate_authentication_headers( $endpoint ) : []
		);

		$body = array_replace_recursive( $body, $this->get_site_info() );

		return $this->request(
			$method,
			$endpoint,
			[
				'timeout' => 100,
				'headers' => $headers,
				'body' => json_encode($body),
			]
		);
	}

	private static function get_remote_url( $endpoint ): string {
		return self::BASE_URL . $endpoint;
	}

	protected function is_connected(): bool {
		return Module::is_connected();
	}

	public function add_bearer_token( $headers ) {
		if ( $this->is_connected() ) {
            $headers['Authorization'] = 'Bearer ' . data::get_access_token();
		}
		return $headers;
	}

	protected function generate_authentication_headers( $endpoint ): array {
		$headers = [
			'endpoint' => $endpoint,
		];

		$headers = $this->add_bearer_token( $headers );

		return $headers;
	}

    protected function request( $method, $endpoint, $args = [] ) {
        $args['method'] = $method;

        if ($method === 'GET' ) {
            $args['body'] = [];
        }

        $args['headers']['Content-Type'] = 'application/json';

        $response = wp_remote_request(
            self::get_remote_url( $endpoint ),
            $args
        );

        if ( is_wp_error( $response ) ) {
            $message = $response->get_error_message();

            return new \WP_Error(
                $response->get_error_code(),
                is_array( $message ) ? join( ', ', $message ) : $message
            );
        }

        $body = wp_remote_retrieve_body( $response );
        $response_code = (int) wp_remote_retrieve_response_code( $response );

        if ( ! $response_code ) {
            return new \WP_Error( 500, 'No Response' );
        }

        // Server sent a success message without content.
        if ( 'null' === $body ) {
            $body = true;
        }

        $body = json_decode( $body );

        if ( false === $body ) {
            return new \WP_Error( 422, 'Wrong Server Response' );
        }

        // If the token is invalid, refresh it and try again once only.
        if ( $this->shouldRefresh($body) ) {
            service::refresh_token();
            $this->refreshed = true;
            $args['headers'] = $this->add_bearer_token( $args['headers'] );
            return $this->request( $method, $endpoint, $args );
        }

        if ( 200 !== (int) $response_code && 201 !== (int) $response_code ) {
            // In case $as_array = true.
            $message = $body->message ?? wp_remote_retrieve_response_message( $response );
            $message = is_array( $message ) ? join( ', ', $message ) : $message;
            $code = isset( $body->code ) ? (int) $body->code : $response_code;

            return new \WP_Error( $code, $message );
        }

        return $body;
    }

    protected function shouldRefresh($body) {
        if ( ! $this->refreshed && ! empty( $body->message ) && ( $body->message === 'Unauthorized' || $body->message === 'Auth Guard Failed!: Invalid Token' ) ) {
            return true;
        }
        else {
            return false;
        }
    }

	/**
	 * get_site_data
	 * @return mixed|\WP_Error  Site data
	 */
	public static function get_site_data() {
		return self::get_instance()->make_request( 'POST', 'send/stats' );
	}

	/**
	 * get_latest_backups
	 * @return mixed|\WP_Error  Backup data
	 */
	public static function get_latest_backups() {
		return self::get_instance()->make_request( 'POST', 'send/latest' );
	}

	/**
	 * register_website
	 * @return mixed|\WP_Error  Site data
	 */
	public static function register_website() {
		return self::get_instance()->make_request( 'POST', 'site' );
	}

	/**
	 * notify_job
	 *
	 * @param string $job_id
	 * @param string $status
	 *
	 * @return mixed|\WP_Error
	 */
	public static function notify_job( string $job_id = '', string $status = '' ) {
		return self::get_instance()->make_request(
			'PATCH',
			'jobs/' . $job_id,
			[ 'currentState' => $status ]
		);

	}
}
