<?php

namespace connect\classes;

use connect\classes\exceptions\Service_Exception;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Class Service
 */
class service
{
    const REFRESH_TOKEN_LOCK = '_connect_refresh_token';

    /**
     * Registers new client and returns client ID
     *
     * @return string
     * @throws Service_Exception
     */
    public static function register_client() {
        $clients_url = Utils::get_clients_url();

        if (!$clients_url) {
           $error = 'Missing clients URL';
            error_log($error);
            return self::handle_error($error);
        }
        $headers = [
            'Content-Type' => 'application/json',
        ];

        $client_data = self::request($clients_url, [
            'method' => 'POST',
            'headers' => $headers,
            'body' => wp_json_encode([
                'redirect_uri' => Utils::get_redirect_uri(),
                'app_type' => Config::APP_TYPE,
            ]),
        ], 201);


        $client_id = $client_data['client_id'] ?? null;
        $client_secret = $client_data['client_secret'] ?? null;

        data::set_client_id($client_id);
        data::set_client_secret($client_secret);
        data::set_auth_redirect_uri();

        return $client_id;
    }

    /**
     * Deactivate license
     *
     * @return void
     * @throws Service_Exception
     */
    public static function deactivate_license()
    {
        $client_id = data::get_client_id();

        if (!$client_id) {
           $error = 'Missing client ID';
            error_log($error);
            return self::handle_error($error);
        }

        $deactivation_url = Utils::get_deactivation_url($client_id);

        if (!$deactivation_url) {
             $error = 'Missing deactivation URL';
             error_log($error);
             return self::handle_error($error);
        }

        $access_token = data::get_access_token();

        if (!$access_token) {
            $error = 'Missing access token';
            error_log($error);
            return self::handle_error($error);
        }

        $refresh_token = data::get_refresh_token();

        if (!$refresh_token) {
            $error = 'Missing refresh token';
            error_log($error);
            return self::handle_error($error);
        }

        self::request($deactivation_url, [
            'method' => 'DELETE',
            'headers' => [
                'Authorization' => "Bearer {$access_token}",
            ],
        ], 204);

        error_log('License deactivated');
        self::get_token(GrantTypes::REFRESH_TOKEN, $refresh_token);
    }

    /**
     * disconnect
     * delete tokens and not client id to force user to re-authenticate with same client-id
     * @return void
     * @throws Service_Exception
     */
    public static function disconnect(): void {
        $sessions_url = Utils::get_sessions_url();

        if (!$sessions_url) {
            $error = 'Missing sessions URL';
            error_log($error);
            self::handle_error($error);
            return;
        }
    $access_token = data::get_access_token();

        if (!$access_token) {
            data::clear_session();
            return;
        }
        $headers = [
            'Content-Type' => 'application/json',
            'Authorization' => "Bearer {$access_token}",
        ];

        try {
            self::request($sessions_url, [
                'method' => 'DELETE',
                'headers' => $headers,
            ], 204);
        }
        catch (Service_Exception $e) {
            error_log($e->getMessage());
        }
        data::clear_session();
    }

    /**
     * Get token & optionally save to user
     *
     * @param string $grant_type
     *
     * @param string|null $credential
     *
     * @return array
     * @throws Service_Exception
     */
    public static function get_token(string $grant_type, ?string $credential = null): array {
        $token_url = Utils::get_token_url();

        if (!$token_url) {
            $error = 'Missing token URL';
            error_log($error);
            self::handle_error($error);
            return [];
        }

        $client_id = data::get_client_id();
        $client_secret = data::get_client_secret();

        if (empty($client_id) || empty($client_secret)) {
            $error = "Missing client ID or secret - ID: {$client_id}, Secret: {$client_secret}";
            error_log($error);
            self::handle_error($error);
            return [];
        }

        $body = [
            'grant_type' => $grant_type,
        ];

        switch ($grant_type) {
            // get authorization code - start concent flow
            case GrantTypes::AUTHORIZATION_CODE:
                $body['code'] = $credential;
                $body['redirect_uri'] = Utils::get_redirect_uri();
                break;
            // get new access and refresh tokens
            case GrantTypes::REFRESH_TOKEN:
                $body[GrantTypes::REFRESH_TOKEN] = $credential;
                break;
            // update credentials on hydra - such as redirect_uri
            case GrantTypes::CLIENT_CREDENTIALS:
                break;
            default:
            throw new Service_Exception( 'Invalid grant type' );
        }

        $headers = [
            'Authorization' => 'Basic ' . base64_encode("{$client_id}:{$client_secret}"),
            'x-elementor-send' => SEND_VERSION,
            'x-elementor-apps' => Config::APP_NAME,
        ];

        $data = self::request($token_url, [
            'method' => 'POST',
            'headers' => $headers,
            'body' => $body,
        ]);

        // handle data have no id_token or access_token or refresh_token
        if ( GrantTypes::AUTHORIZATION_CODE || GrantTypes::REFRESH_TOKEN) {
            try {
                if (!isset($data['id_token']) || !isset($data['access_token']) || !isset($data['refresh_token'])) {
                    $error = 'Missing token data';
                    throw new Service_Exception($error);
                }

                data::set_connect_mode_data(data::TOKEN_ID, $data['id_token']);
                data::set_connect_mode_data(data::ACCESS_TOKEN, $data['access_token']);
                data::set_connect_mode_data(data::REFRESH_TOKEN, $data['refresh_token']);
            } catch (\Throwable $th) {
                error_log($th->getMessage());
            }
        }

        return $data;
    }

    public static function jwt_decode($payload): string
    {
        static $jwks = null;

        $jwks_url = Utils::get_jwks_url();
        if (!$jwks_url) {
            return __('Missing JWKS URL');
        }

        if (!$jwks) {
            $jwks = self::request($jwks_url, [
                'method' => 'GET',
            ]);
        }
        if (!class_exists('JWT')) {
            require_once SEND_PLUGIN_DIR . 'vendor/autoload.php';
        }

        try {
            $decoded = \Firebase\JWT\JWT::decode($payload, \Firebase\JWT\JWK::parseKeySet($jwks));
            return json_encode($decoded, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);
        } catch (\Throwable $th) {
            if ($th instanceof \Firebase\JWT\ExpiredException) {
                error_log('Token expired');
                self::get_token(GrantTypes::REFRESH_TOKEN, data::get_refresh_token());
                return self::jwt_decode($payload);
            }
            return $th->getMessage();
        }
    }

    	/**
	 * @param string $url
	 * @param array $args
	 * @param int $valid_response_code
	 *
	 * @return array|null
	 * @throws Service_Exception
	 */
	public static function request( string $url, array $args, int $valid_response_code = 200 ): ?array {
		$args['timeout'] = 30;

		$response = wp_remote_request( $url, $args );

		if ( is_wp_error( $response ) ) {
			error_log( '[Connect Service]: ' . $response->get_error_message() );
			throw new Service_Exception( esc_html( $response->get_error_message() ) );
		}

		if ( wp_remote_retrieve_response_code( $response ) !== $valid_response_code ) {
			error_log( '[Connect Service]: invalid status code ' . wp_remote_retrieve_response_code( $response ) );
			throw new Service_Exception( esc_html( wp_remote_retrieve_body( $response ) ) );
		}

		return json_decode( wp_remote_retrieve_body( $response ), true );
	}

    static function handle_error($message, $error_code = '') {
        $class = 'notice notice-error';

        // Append error code to the message if provided
        $full_message = __( 'An error has occurred. Please try again!', 'sample-text-domain' );
        if ($error_code) {
            $full_message .= ' (' . esc_html($error_code) . ')';
        }

        // Add the specific error message, if any
        if (!empty($message)) {
            $full_message .= ' ' . esc_html($message);
        }

        printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $full_message ) );
    }


    public static function refresh_token() {
        $lock_key = Config::APP_NAME . self::REFRESH_TOKEN_LOCK;
        $last_token = data::fetch_option($lock_key, '');

        $current_refresh_token = data::get_refresh_token();
        if (!empty($last_token) && $last_token === $current_refresh_token) {
            sleep(1);
            // TODO: add log to identify if happens - add tests.
            return;
        }

        delete_option($lock_key);
        $locked = data::insert_option_uniquely($lock_key, $current_refresh_token);
        if (!$locked) {
            sleep(1);
            return;
        }
        try {
            $response = self::get_token(GrantTypes::REFRESH_TOKEN, $current_refresh_token);
        } catch (Service_Exception $e) {
            $error_message = $e->getMessage();
            delete_option($lock_key);
            data::clear_tokens();
            wp_redirect(Utils::get_redirect_uri());
        }
    }

    /**
	 * @throws Service_Exception
	 */
	public static function update_redirect_uri(): void {
		$client_id = Data::get_client_id();

		if ( ! $client_id ) {
			throw new Service_Exception( 'Missing client ID' );
		}

		$client_patch_url = Utils::get_clients_patch_url( $client_id );

		[ 'access_token' => $access_token ] = self::get_token(
			GrantTypes::CLIENT_CREDENTIALS,
			null,
			false
		);

		self::request( $client_patch_url, [
			'method' => 'PATCH',
			'headers' => [
				'Content-Type' => 'application/json',
				'Authorization' => "Bearer {$access_token}",
			],
			'body' => wp_json_encode( [
				'redirect_uri' => Utils::get_redirect_uri(),
			] ),
		] );
        // TODO: consider removing this
		self::refresh_token();

		data::set_auth_redirect_uri();
	}
}
