<?php

use connect\classes\Config;
use connect\classes\data;
use connect\classes\exceptions\Service_Exception;
use connect\classes\service;
use connect\classes\Utils;
use connect\Module as ConnectSDK;
use Send\Classes\Integrations\Integrations;
use Send\Classes\Logger\Logger;
use Send\Classes\Traits\Connect;

require_once 'vendor/autoload.php';
require_once("app/Classes/Database/Database.php");
require_once plugin_dir_path(__FILE__) . 'app/Config/Constants.php';

class Send
{

    public static $pluginDir = __FILE__;
    private static $instance = null;
    /**
     * @var string
     */
    private static $accessToken;
    public $integrations;
    private $scriptsLoader;
    private $initiated = false;

    private static $lastRun = null;
    private $interval;


    public function __construct() {
	    $this->includes();

        $this->interval = 1.5; // 1.5 seconds
        add_action( 'admin_init', [ $this, 'onAdminInit' ] );
        add_action('init', [$this, 'init']);
        add_action('activated_plugin', [$this, 'listenForPluginsActivation']);
        add_action('admin_menu', [$this, 'add_menu']);
        self::loadEnv();
    }

    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }


    public function init()
    {
        // Added check for compatibility
        $this->check_compatibility();

        if (!function_exists('get_plugin_data')) {
            require_once(ABSPATH . 'wp-admin/includes/plugin.php');
        }

        if ($this->initiated) {

            return;
        };

        $this->scriptsLoader = new Send\Classes\Scripts\ScriptsLoader();
        $this->useApi();
        $this->initiated = true;

        $this->loadIntegrations();

        // Check if plugin is already connected
        // if not connected and not admin - stop the process to avoid breaking the site.
        if (!is_admin() && !ConnectSDK::is_connected()) {
            // TODO: handle the case when the plugin is not connected and the user is not admin with some message queue to prevent data loss
            return;
        }

    }

    private static function loadEnv() {
        switch (ENV) {
            case 'prod':
                require_once 'app/Config/Prod.php';
                break;
            case 'stg':
                require_once 'app/Config/Stg.php';
                break;
            case 'dev':
                require_once 'app/Config/Dev.php';
                break;
            case 'local':
                require_once 'app/Config/Local.php';
                break;
        }
    }
    // TODO: move to authorize or connect class
    private static function authorize() {
        if (!Connect::should_show_connect() || ConnectSDK::is_connected()) {
            return;
        }

        // try to get client-id from options db
        $client_id = data::get_client_id();
            // user never registered client - only on first entrance or after manual deactivation
            if (!$client_id) {
                try {
                    $client_id = service::register_client();
                    if (!isset($client_id)) {
                        // TODO: consider return Service_Exception or wp_die
                        service::handle_error("Client registration failed". "Please contact support");
                        return;
                    }
                } catch (Service_Exception $e) {
                    // TODO: consider return Service_Exception or wp_die
                    service::handle_error("Client registration failed". "Please contact support");
                    error_log("Client registration failed: " . $e->getMessage());
                    return;
                }
            }
        // client-id exists or just registered - and have no tokens - redirect to auth consent page
        $authorize_url = Utils::get_authorize_url($client_id);
        wp_redirect($authorize_url);
        exit;
    }

    private function validate_nonce( $state ) {
        if ( ! wp_verify_nonce( $state, Config::STATE_NONCE ) ) {
            wp_die( 'Invalid state' );
        }
    }

    public function listenForPluginsActivation($plugin)
    {
        $pluginBaseName = plugin_basename($plugin);
        $pluginName = $this->get_plugin_name($pluginBaseName);
    }


    public function get_plugin_name($plugin)
    {
        // Get plugin name from plugin file header
        $data = get_file_data(WP_PLUGIN_DIR . '/' . $plugin, array('Name' => 'Plugin Name'));
        return $data['Name'];
    }

    private function useApi()
    {
        require_once 'app/routes/Api.php';
    }

    public function pluginActionLinks($links)
    {
        $settings_link = '<a href="admin.php?page=' . Config::ADMIN_PAGE . '">Overview</a>';
        array_unshift($links, $settings_link); // Add the link to the beginning of the array
        return $links;
    }
    // Activation hook
    public function activate()
    {
        Logger::log('Plugin activated', 'debug');
        $this->init();
        if (ConnectSDK::is_connected()) {
            Logger::log('Plugin activated and connected - syncing data', 'debug');
            Integrations::sendCreateIntegrationsData();
            Integrations::sendFormsData();
        }
    }

    // Add menu item to the admin menu
    public function add_menu()
    {
        $this->check_compatibility(); // Added check for compatibility
        $pluginIcon = plugins_url('/app/Assets/images/send-menu.svg', __FILE__);
        $textToAdd = '';
        if (ENV !== 'prod') {
            $textToAdd = ' - ' . ENV;
        }
        Logger::log('Adding menu', 'debug', ['textToAdd' => $textToAdd, 'pluginIcon' => $pluginIcon, 'adminPage' => Config::ADMIN_PAGE]);
        add_menu_page(
            'Send' . $textToAdd,                                        // Page title
            'Send' . $textToAdd,                                        // Menu title
            'manage_options',                             // Capability required to access
            Config::ADMIN_PAGE,                      // Menu slug
            [$this, 'displayPage'],                       // Callback function to display the page content
            $pluginIcon, // URL of the custom icon
            45                                            // Position of the menu item
        );
    }

    // Callback function to display the page content
    public function displayPage() {
        // if not connected - redirect to auth consent page
        if (!ConnectSDK::is_connected()) {
            // consider move to other function
            // if user !clientId log - we must know about that.
            $client_id = data::get_client_id();
            $authorize_url = Utils::get_authorize_url($client_id);
            wp_redirect($authorize_url);
            exit;
        } else {
            $this->loadScripts();
            echo '<div class="app"></div>';
        }
    }

    public function loadScripts()
    {
        $this->scriptsLoader->loadAppScripts();
    }

    public function loadIntegrations()
    {
        $this->scriptsLoader->loadTrackings();
        $this->initIntegrations();
        $this->registerIntegrationsSyncHooks();
    }

    public function initIntegrations()
    {
        if ($this->integrations) {
            foreach ($this->integrations as $key => $integration) {
                Logger::log('Initiating integration', 'debug', ['key' => $key]);
                if (is_object($integration)) {
                    Logger::log('Integration already exists', 'debug', ['key' => $key]);
                    $this->integrations[$key] = null;
                }
            }
        }
        $this->integrations['cf7'] = \Send\Classes\Integrations\Forms\Cf7::getInstance();
        $this->integrations['wpforms'] = \Send\Classes\Integrations\Forms\WpForms::getInstance();
        $this->integrations['elementor'] = \Send\Classes\Integrations\Forms\ElementorForms::getInstance();
        $this->integrations['woocommerce'] = \Send\Classes\Integrations\WooCommerce\WooCommerce::getInstance();
    }

    private function registerIntegrationsSyncHooks()
    {
          add_action('upgrader_process_complete', [$this, 'onAnyPluginInstalled'], 10, 2);
          add_action('deleted_plugin', [$this, 'onAnyPluginUninstalled'], 10, 2);
    }

    public function onAdminInit () {
        if (is_admin()) {
            // register connect hooks
            $this->registerConnectHooks();
        }
        // if admin and not connected - handle the connect
        if (is_admin() && !ConnectSDK::is_connected()) {
            // if returned from concent will handle auth and trigger onConnect function
            Connect::handle_auth_code();

            // if not connected and skipped the handle_auth_code - need to authorize
            return self::authorize();
        }

        // if connected - make sure the redirect_uri is valid and refresh tokens
        if (is_admin() && ConnectSDK::is_connected() && !Utils::is_valid_auth_redirect_uri()) {
            // TODO: need to enrich the monitoring and logging for this case
            service::update_redirect_uri();
        }
    }

    private function registerConnectHooks() {
        add_action( 'on_connect_' . Config::APP_PREFIX . '_connected', [ $this, 'onConnect' ] );
    }

    public function onConnect() {
		if ( ! ConnectSDK::is_connected() ) {
			return;
		}
        try {
            // TODO: skip if site is already registered
            Connect::register_site_on_collection(data::get_access_token());
            // TODO: add oops error, add check id site exists on init - if site not exists - if client id and access_token call to OnConnect
            Integrations::sendCreateIntegrationsData();
            Integrations::sendFormsData();
        } catch (Exception $e) {
            wp_die('Error registering site on collection' . $e->getMessage());
        }
	}

    function onAnyPluginInstalled($plugin, $options)
    {
        if ($options['type'] === 'plugin' && $options['action'] === 'install') {
            Integrations::sendInstalledOnlyIntegrationsData();
        }
    }

    function onAnyPluginUninstalled($plugin, $deleted)
    {
        // Check if the plugin was successfully deleted
        if ($deleted) {
            foreach ($this->integrations as $key => $integration) {
                if ($integration->pluginFile === $plugin) {
                    Integrations::deletePlugin($integration->sourceName);
                }
            }
        }
    }

    // Function to check compatibility with WordPress and PHP versions
    private function check_compatibility()
    {
        // WordPress version check
        global $wp_version;
        Logger::log('Checking compatibility', 'debug', ['wp_version' => $wp_version, 'min_version' => WP_MIN_VERSION]);
        $requiredWpVersion = WP_MIN_VERSION; // minimum required WordPress version
        if (version_compare($wp_version, $requiredWpVersion, '<')) {
            Logger::log('WordPress version is not compatible', 'error', ['wp_version' => $wp_version, 'min_version' => WP_MIN_VERSION]);
            deactivate_plugins(plugin_basename(__FILE__));
            wp_die(esc_html("This plugin requires WordPress version {$requiredWpVersion} or higher."));
        }
        // PHP version check
        Logger::log('Checking PHP version', 'debug', ['php_version' => phpversion(), 'min_version' => PHP_MIN_VERSION]);
        $requiredPhpVersion = PHP_MIN_VERSION; // minimum required PHP version
        if (version_compare(phpversion(), $requiredPhpVersion, '<')) {
            Logger::log('PHP version is not compatible', 'error', ['php_version' => phpversion(), 'min_version' => PHP_MIN_VERSION]);
            deactivate_plugins(plugin_basename(__FILE__));
            wp_die(esc_html("This plugin requires PHP version {$requiredPhpVersion} or higher."));
        }
    }

    public function updateLastRun()
    {
        self::$lastRun = microtime(true); // Updating with the current time
    }

	private function includes() {
		require plugin_dir_path( __FILE__ ) . 'app/Classes/includes/modules-manager.php';
	}
}
